@@ -53,6 +53,7 @@ protected:
 private:
 	bool need_redirect();
 
+	std::string username_;
 	std::string password_;
 	int db_num_;
 	bool succ_;
@@ -83,25 +84,31 @@ CommMessageOut *ComplexRedisTask::message_out()
 
 	if (seqid <= 1)
 	{
-		if (seqid == 0 && !password_.empty())
+		if (seqid == 0 && (!password_.empty() || !username_.empty()))
 		{
-			succ_ = false;
-			is_user_request_ = false;
 			auto *auth_req = new RedisRequest;
 
-			auth_req->set_request("AUTH", {password_});
+			if (!username_.empty())
+				auth_req->set_request("AUTH", {username_, password_});
+			else
+				auth_req->set_request("AUTH", {password_});
+
+			succ_ = false;
+			is_user_request_ = false;
 			return auth_req;
 		}
 
-		if (db_num_ > 0 && (seqid == 0 || !password_.empty()))
+		if (db_num_ > 0 &&
+			(seqid == 0 || !password_.empty() || !username_.empty()))
 		{
-			succ_ = false;
-			is_user_request_ = false;
 			auto *select_req = new RedisRequest;
 			char buf[32];
 
 			sprintf(buf, "%d", db_num_);
 			select_req->set_request("SELECT", {buf});
+
+			succ_ = false;
+			is_user_request_ = false;
 			return select_req;
 		}
 	}
@@ -137,7 +144,7 @@ int ComplexRedisTask::keep_alive_timeout()
 
 bool ComplexRedisTask::init_success()
 {
-	TransportType type;
+	enum TransportType type;
 
 	if (uri_.scheme && strcasecmp(uri_.scheme, "redis") == 0)
 		type = TT_TCP;
@@ -159,18 +166,26 @@ bool ComplexRedisTask::init_success()
 		char *p = strchr(uri_.userinfo, ':');
 		if (p)
 		{
+			username_.assign(uri_.userinfo, p);
 			password_.assign(p + 1);
+			StringUtil::url_decode(username_);
 			StringUtil::url_decode(password_);
 		}
+		else
+		{
+			username_.assign(uri_.userinfo);
+			StringUtil::url_decode(username_);
+		}
 	}
 
 	if (uri_.path && uri_.path[0] == '/' && uri_.path[1])
 		db_num_ = atoi(uri_.path + 1);
 
-	size_t info_len = password_.size() + 32 + 16;
+	size_t info_len = username_.size() + password_.size() + 32 + 32;
 	char *info = new char[info_len];
 
-	sprintf(info, "redis|pass:%s|db:%d", password_.c_str(), db_num_);
+	sprintf(info, "redis|user:%s|pass:%s|db:%d", username_.c_str(),
+			password_.c_str(), db_num_);
 	this->WFComplexClientTask::set_transport_type(type);
 	this->WFComplexClientTask::set_info(info);
 
