@@ -625,6 +625,13 @@ public:
 		return *this;
 	}
 
+	KafkaConfig(const KafkaConfig& copy)
+	{
+		this->ptr = copy.ptr;
+		this->ref = copy.ref;
+		++*this->ref;
+	}
+
 	KafkaConfig& operator= (const KafkaConfig& copy)
 	{
 		if (this != &copy)
@@ -634,6 +641,7 @@ public:
 			this->ref = copy.ref;
 			++*this->ref;
 		}
+
 		return *this;
 	}
 
@@ -731,7 +739,14 @@ public:
 		return *this;
 	}
 
-	KafkaRecord& operator= (KafkaRecord& copy)
+	KafkaRecord(const KafkaRecord& copy)
+	{
+		this->ptr = copy.ptr;
+		this->ref = copy.ref;
+		++*this->ref;
+	}
+
+	KafkaRecord& operator= (const KafkaRecord& copy)
 	{
 		if (this != &copy)
 		{
@@ -740,6 +755,7 @@ public:
 			this->ref = copy.ref;
 			++*this->ref;
 		}
+
 		return *this;
 	}
 
@@ -866,7 +882,7 @@ public:
 		this->endpos = copy.endpos;
 	}
 
-	KafkaToppar& operator= (KafkaToppar& copy)
+	KafkaToppar& operator= (const KafkaToppar& copy)
 	{
 		if (this != &copy)
 		{
@@ -878,6 +894,7 @@ public:
 			this->startpos = copy.startpos;
 			this->endpos = copy.endpos;
 		}
+
 		return *this;
 	}
 
@@ -999,14 +1016,12 @@ public:
 		return this->ptr->port;
 	}
 
-	std::string get_uri() const
+	std::string get_host_port() const
 	{
-		std::string uri = "kafka://";
-
-		uri += this->ptr->host;
-		uri += ":";
-		uri += std::to_string(this->ptr->port);
-		return uri;
+		std::string host_port(this->ptr->host);
+		host_port += ":";
+		host_port += std::to_string(this->ptr->port);
+		return host_port;
 	}
 
 	int get_error()
@@ -1086,12 +1101,12 @@ public:
 
 	bool operator< (const KafkaBroker& broker) const
 	{
-		return this->get_uri() < broker.get_uri();
+		return this->get_host_port() < broker.get_host_port();
 	}
 
 	bool operator> (const KafkaBroker& broker) const
 	{
-		return this->get_uri() > broker.get_uri();
+		return this->get_host_port() > broker.get_host_port();
 	}
 
 	kafka_broker_t *get_raw_ptr() const { return this->ptr; }
@@ -1100,54 +1115,6 @@ public:
 
 	struct rb_node *get_rb() { return &this->rb; }
 
-	bool is_equal(const struct sockaddr *addr, socklen_t socklen) const
-	{
-		if (this->ptr->addrlen == socklen && socklen)
-			return memcmp(addr, &this->ptr->addr, this->ptr->addrlen) == 0;
-
-		return false;
-	}
-
-	bool is_equal(const char *host, int port) const
-	{
-		if (port == this->ptr->port)
-			return strcmp(host, this->ptr->host) == 0;
-		return false;
-	}
-
-	bool is_equal(int node_id) const
-	{
-		return this->ptr->node_id == node_id;
-	}
-
-	bool is_equal(const KafkaBroker& broker) const
-	{
-		return is_equal(broker.ptr->host, broker.ptr->port);
-	}
-
-	void get_broker_addr(const struct sockaddr **addr, socklen_t *socklen) const
-	{
-		if (this->ptr->addrlen)
-		{
-			*addr = (const struct sockaddr *)&this->ptr->addr;
-			*socklen = this->ptr->addrlen;
-		}
-		else
-		{
-			*addr = NULL;
-			*socklen = 0;
-		}
-	}
-
-	void set_broker_addr(const struct sockaddr *addr, socklen_t socklen)
-	{
-		memcpy(&this->ptr->addr, addr, socklen);
-		this->ptr->addrlen = socklen;
-	}
-
-	bool is_to_addr() const { return this->ptr->to_addr == 1; }
-	void set_to_addr(int to_addr) { this->ptr->to_addr = to_addr; }
-
 	int get_node_id() const { return this->ptr->node_id; }
 
 	int get_id () const { return this->ptr->node_id; }
@@ -1234,7 +1201,7 @@ public:
 		++*this->ref;
 	}
 
-	KafkaMeta& operator= (KafkaMeta& copy)
+	KafkaMeta& operator= (const KafkaMeta& copy)
 	{
 		if (this != &copy)
 		{
@@ -1243,6 +1210,7 @@ public:
 			this->ref = copy.ref;
 			++*this->ref;
 		}
+
 		return *this;
 	}
 
@@ -1376,14 +1344,6 @@ public:
 		return strcmp(this->ptr->leader_id, this->ptr->member_id) == 0;
 	}
 
-	bool is_equal_coordinator(const struct sockaddr *addr, socklen_t addrlen) const
-	{
-		if (addrlen == this->ptr->coordinator.addrlen)
-			return memcmp(addr, &this->ptr->coordinator.addr, addrlen) == 0;
-		else
-			return false;
-	}
-
 	struct list_head *get_group_protocol()
 	{
 		return &this->ptr->group_protocol_list;
