@@ -55,6 +55,22 @@ private:
 	std::vector<UPSGroupPolicy *> upstream_policies;
 };
 
+int UpstreamManager::upstream_create_round_robin(const std::string& name,
+												 bool try_another)
+{
+	WFNameService *ns = WFGlobal::get_name_service();
+	auto *policy = new UPSRoundRobinPolicy(try_another);
+
+	if (ns->add_policy(name.c_str(), policy) >= 0)
+	{
+		__UpstreamManager::get_instance()->add_upstream_policy(policy);
+		return 0;
+	}
+
+	delete policy;
+	return -1;
+}
+
 static unsigned int __default_consistent_hash(const char *path,
 											  const char *query,
 											  const char *fragment)
@@ -70,7 +86,7 @@ static unsigned int __default_consistent_hash(const char *path,
 int UpstreamManager::upstream_create_consistent_hash(const std::string& name,
 													 upstream_route_t consistent_hash)
 {
-	auto *ns = WFGlobal::get_name_service();
+	WFNameService *ns = WFGlobal::get_name_service();
 	UPSConsistentHashPolicy *policy;
 
 	policy = new UPSConsistentHashPolicy(
@@ -89,8 +105,8 @@ int UpstreamManager::upstream_create_consistent_hash(const std::string& name,
 int UpstreamManager::upstream_create_weighted_random(const std::string& name,
 													 bool try_another)
 {
-	auto *ns = WFGlobal::get_name_service();
-	UPSWeightedRandomPolicy *policy = new UPSWeightedRandomPolicy(try_another);
+	WFNameService *ns = WFGlobal::get_name_service();
+	auto *policy = new UPSWeightedRandomPolicy(try_another);
 
 	if (ns->add_policy(name.c_str(), policy) >= 0)
 	{
@@ -104,8 +120,8 @@ int UpstreamManager::upstream_create_weighted_random(const std::string& name,
 
 int UpstreamManager::upstream_create_vnswrr(const std::string& name)
 {
-	auto *ns = WFGlobal::get_name_service();
-	UPSWeightedRandomPolicy *policy = new UPSVNSWRRPolicy();
+	WFNameService *ns = WFGlobal::get_name_service();
+	auto *policy = new UPSVNSWRRPolicy();
 
 	if (ns->add_policy(name.c_str(), policy) >= 0)
 	{
@@ -122,7 +138,7 @@ int UpstreamManager::upstream_create_manual(const std::string& name,
 											bool try_another,
 											upstream_route_t consistent_hash)
 {
-	auto *ns = WFGlobal::get_name_service();
+	WFNameService *ns = WFGlobal::get_name_service();
 	UPSManualPolicy *policy;
 
 	policy = new UPSManualPolicy(try_another, std::move(select),
