@@ -61,14 +61,6 @@ static constexpr struct AddressParams ADDRESS_PARAMS_DEFAULT =
 	.group_id			=	-1,
 };
 
-enum ServerChangeState
-{
-	ADD_SERVER		=	0,
-	REMOVE_SERVER	=	1,
-	RECOVER_SERVER	=	2,
-	FUSE_SERVER		=	3,
-};
-
 class PolicyAddrParams
 {
 public:
@@ -77,8 +69,10 @@ public:
 	unsigned int dns_ttl_min;
 	unsigned int max_fails;
 
+public:
 	PolicyAddrParams();
 	PolicyAddrParams(const struct AddressParams *params);
+	virtual ~PolicyAddrParams() { }
 };
 
 class EndpointAddress
@@ -124,10 +118,6 @@ public:
 	void enable_server(const std::string& address);
 	void disable_server(const std::string& address);
 	virtual void get_current_address(std::vector<std::string>& addr_list);
-	virtual void server_list_change(const EndpointAddress *address,
-									enum ServerChangeState state)
-	{
-	}
 
 	void set_mttr_second(unsigned int second) { this->mttr_second = second; }
 	static bool in_select_history(WFNSTracing *tracing, EndpointAddress *addr);
