@@ -22,7 +22,6 @@
 
 #define DNS_LABELS_MAX				63
 #define DNS_MESSAGE_MAX_UDP_SIZE	512
-#define DNS_HEADER_SIZE				sizeof (struct dns_header)
 
 namespace protocol
 {
@@ -77,7 +76,6 @@ int DnsMessage::encode_reply()
 	size_t len;
 
 	msgbuf.clear();
-	msgbuf.reserve(DNS_HEADER_SIZE);
 	msgsize = 0;
 
 	// TODO encode other field
