@@ -1562,44 +1562,6 @@ int KafkaMessage::parse_records(void **buf, size_t *size, bool check_crcs,
 	return 0;
 }
 
-static bool __to_addr(const char *host, int port, struct sockaddr *sockaddr,
-					  socklen_t *addrlen)
-{
-	size_t len = strlen(host);
-	struct sockaddr_in *addr;
-	struct sockaddr_in6 *addr6;
-	bool ret = true;
-
-	if (!host)
-		ret = false;
-	else if (isdigit(host[0]) && isdigit(host[len - 1]))
-	{
-		addr = (struct sockaddr_in *)sockaddr;
-		if (inet_pton(AF_INET, host, &addr->sin_addr) == 1)
-		{
-			addr->sin_family = AF_INET;
-			*addrlen = sizeof(struct sockaddr_in);
-			addr->sin_port = htons(port);
-		}
-		else
-			ret = false;
-	}
-	else
-	{
-		addr6 = (struct sockaddr_in6 *)sockaddr;
-		if (inet_pton(AF_INET6, host, &addr6->sin6_addr) == 1)
-		{
-			addr6->sin6_family = AF_INET6;
-			*addrlen = sizeof(struct sockaddr_in6);
-			addr6->sin6_port = htons(port);
-		}
-		else
-			ret = false;
-	}
-
-	return ret;
-}
-
 KafkaMessage::KafkaMessage()
 {
 	static struct Crc32cInitializer
@@ -2867,11 +2829,6 @@ static int kafka_meta_parse_broker(void **buf, size_t *size,
 		if (api_version >= 1)
 			CHECK_RET(parse_string(buf, size, &ptr->rack));
 
-		if (__to_addr(ptr->host, ptr->port, (struct sockaddr *)&ptr->addr, &ptr->addrlen))
-		{
-			ptr->to_addr = 1;
-		}
-
 		broker_list->rewind();
 		KafkaBroker *last;
 
@@ -3364,12 +3321,6 @@ int KafkaResponse::parse_findcoordinator(void **buf, size_t *size)
 	CHECK_RET(parse_string(buf, size, &cgroup->coordinator.host));
 	CHECK_RET(parse_i32(buf, size, &cgroup->coordinator.port));
 
-	if (__to_addr(cgroup->coordinator.host, cgroup->coordinator.port,
-				   (struct sockaddr *)&cgroup->coordinator.addr, &cgroup->coordinator.addrlen))
-	{
-		cgroup->coordinator.to_addr = 1;
-	}
-
 	return 0;
 }
 
