@@ -51,19 +51,43 @@ json_array_t *json_value_array(const json_value_t *val);
 const json_value_t *json_object_find(const char *name,
 									 const json_object_t *obj);
 int json_object_size(const json_object_t *obj);
-const char *json_object_next_name(const char *prev,
+const char *json_object_next_name(const char *name,
 								  const json_object_t *obj);
-const json_value_t *json_object_next_value(const json_value_t *prev,
+const json_value_t *json_object_next_value(const json_value_t *val,
+										   const json_object_t *obj);
+const char *json_object_prev_name(const char *name,
+								  const json_object_t *obj);
+const json_value_t *json_object_prev_value(const json_value_t *val,
 										   const json_object_t *obj);
 const json_value_t *json_object_append(json_object_t *obj,
 									   const char *name,
 									   int type, ...);
+const json_value_t *json_object_insert_after(const json_value_t *val,
+											 json_object_t *obj,
+											 const char *name,
+											 int type, ...);
+const json_value_t *json_object_insert_before(const json_value_t *val,
+											  json_object_t *obj,
+											  const char *name,
+											  int type, ...);
+json_value_t *json_object_remove(const json_value_t *val,
+								 json_object_t *obj);
 
 int json_array_size(const json_array_t *arr);
 const json_value_t *json_array_next_value(const json_value_t *val,
 										  const json_array_t *arr);
+const json_value_t *json_array_prev_value(const json_value_t *val,
+										  const json_array_t *arr);
 const json_value_t *json_array_append(json_array_t *arry,
 									  int type, ...);
+const json_value_t *json_array_insert_after(const json_value_t *val,
+											json_array_t *arr,
+											int type, ...);
+const json_value_t *json_array_insert_before(const json_value_t *val,
+											 json_array_t *arr,
+											 int type, ...);
+json_value_t *json_array_remove(const json_value_t *val,
+								json_array_t *arr);
 
 #ifdef __cplusplus
 }
@@ -74,8 +98,16 @@ const json_value_t *json_array_append(json_array_t *arry,
 		 name = json_object_next_name(name, obj), \
 		 val = json_object_next_value(val, obj), val; )
 
+#define json_object_for_each_prev(name, val, obj) \
+	for (name = NULL, val = NULL; \
+		 name = json_object_prev_name(name, obj), \
+		 val = json_object_prev_value(val, obj), val; )
+
 #define json_array_for_each(val, arr) \
 	for (val = NULL; val = json_array_next_value(val, arr), val; )
 
+#define json_array_for_each_prev(val, arr) \
+	for (val = NULL; val = json_array_prev_value(val, arr), val; )
+
 #endif
 
